#if !defined(_EXPECT_PROCESS_H_)
#define _PIPE_PROCESS_H_


/*****************************************************************************
		       Copyright (c) John F. Croix, 1997

			      All Rights Reserved

  Permission to use, copy, modify, and distribute this software and its
  documentation for any non-commercial or internal-use-only purpose and without
  fee is hereby granted, provided that the above copyright notice appears in
  all copies and that both the copyright notice and this permission notice
  appear in supporting documentation, and all changes or alterations to the
  software and documentation are clearly documented.  Resale of any part of
  this software or documentation is prohibited without a license.  Licenses may
  be obtained by contacting the author, John F. Croix, at John.Croix@amd.com or
  journeyman@mail.utexas.edu.

  THE AUTHOR, JOHN F. CROIX, DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
  SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
  DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
  TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OF PERFORMANCE
  OF THIS SOFTWARE.
 *****************************************************************************/


#include "accurate_time.h"
#include "pipe_pty.h"
#include <setjmp.h>
#include <stdio.h>
#include <sys/types.h>


/*****************************************************************************
 * This class is responsible for handling all interaction with a child
 * process.  It is responsible for creating, maintaining, closing, and
 * interacting with the child process.  To make life easy, it uses the
 * Expect library where it makes sense.
 *****************************************************************************/


class PipeProcess
  {
  public:
    enum ErrorStatus { OK=0, EOF_FOUND, ERROR_FOUND, LOG_ERROR,
                       PTY_ERROR, RANGE_ERROR, TIMEOUT };

  private:
    enum { DEFAULT_BUFFER_SIZE=16384, DEFAULT_TIMEOUT=10 };

    PipePTY PTY;
    ErrorStatus Status;
    int ReadTimeout;
    unsigned BufferCursor, BufferSize, BufferLimit, ErrorCode;
    char *Buffer, *LineBuffer;
    AccurateTime Timer;
    FILE *LogFile;

    /*
     * Private methods
     */

    static void AlarmHandler(int);
    void Init();
    static jmp_buf &JumpBuffer();
    ssize_t ReadFromProcess(unsigned CharsToRead, char *Buffer);

  public:
    PipeProcess();
    PipeProcess(const char *Process, const char *Arguments[]);
    PipeProcess(int FileDescriptor);
    ~PipeProcess();

    /*
     * Buffer manipulation
     */

    void Clear();
    const char *Line(int Echo);
    const char *Data() const;
    int FullBuffer() const;
    unsigned Length() const;
    void RemoveNulls();
    unsigned Size() const;
    unsigned Size(unsigned NewSize);
    unsigned Splice(unsigned From, unsigned To, char *SpliceBuffer=0);
  
    /*
     * Process communication
     */

    void Close();
    void Open(int FileDescriptor);
    void Open(const char *Process, const char *Arguments[]);
    unsigned Read(int Echo);
    void STTY(const char *Command);
    int Timeout() const;
    int Timeout(int NewTimeout);
    int Wait();
    unsigned Write(const char *Message, unsigned Len=0);

    /*
     * Misc
     */

    void CloseLog();
    int OpenLog(const char *Name, const char *Mode);
    void ResetTimeoutTimer();

    /*
     * Error handling
     */

    ErrorStatus ErrorValue() const;
    const char *ErrorMsg() const;
    void ResetError();
  };


#endif

