#if !defined(_PIPE_PTY_H_)
#define _PIPE_PTY_H_


/*****************************************************************************
		       Copyright (c) John F. Croix, 1997

			      All Rights Reserved

  Permission to use, copy, modify, and distribute this software and its
  documentation for any non-commercial or internal-use-only purpose and without
  fee is hereby granted, provided that the above copyright notice appears in
  all copies and that both the copyright notice and this permission notice
  appear in supporting documentation, and all changes or alterations to the
  software and documentation are clearly documented.  Resale of any part of
  this software or documentation is prohibited without a license.  Licenses may
  be obtained by contacting the author, John F. Croix, at John.Croix@amd.com or
  journeyman@mail.utexas.edu.

  THE AUTHOR, JOHN F. CROIX, DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
  SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
  DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
  TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OF PERFORMANCE
  OF THIS SOFTWARE.
 *****************************************************************************/


#include <sys/types.h>
#include <sys/wait.h>
#include <termios.h>


/*****************************************************************************
 * The purpose of this class is to provide a mechanism through which a process
 * can be spawned and communicated with.
 *
 * Much of the work was pioneered by Don Libes (libes@nist.gov) of Expect
 * fame.
 *****************************************************************************/


class PipePTY
  {
  public:
    enum ErrorStatus { OK=0, BAD_SPAWN, BAD_STTY, PTY_CANNOT_WAIT,
		       PTY_CLOSED, PTY_UNINITIALIZED, SYSTEM_ERROR,
                       BAD_FILE_DES };

  private:
    char *STTYCommand;
    int SpawnID, SystemErrno;
    pid_t ChildID;
    ErrorStatus Status;
    termios *TermState;

  public:
    PipePTY();
    ~PipePTY();

    /*
     * Subprocess handling
     */

    void Close();
    int ID() const;
    int IsProcess() const;
    ErrorStatus Open(int FileDescriptor);
    ErrorStatus Open(const char *Program, const char *Arguments[]);
    void STTY(const char *Command);
    int Wait();

    /*
     * Error handling
     */

    ErrorStatus ErrorValue() const;
    const char *ErrorMsg() const;
    void ErrorReset();
  };


#endif


